#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include "farmaco.h"
#include "readField.h"

#define SEPARATOR ';'

Farmaco* createFarmaco()
{
	Farmaco *f = (Farmaco*)malloc(sizeof(Farmaco));
	f->nome[0] = '\0';
	f->originale = false;
	f->principioAttivo.nome[0] = '\0';
	f->principioAttivo.dosaggio = 0;
	f->produttore[0] = '\0';
	f->giacenza = 0;
	return f;
}

void destroyFarmaco(Farmaco *f)
{
	free(f);
}

/*
Formato File
Nome Farmaco;O/G;Principio Attivo;dosaggio;Produttore ...;giacenza
*/

Farmaco* readFarmacoFromTxt(FILE *file)
{
	Farmaco *farmaco;
	Boolean ok;
	char originale;

	farmaco = createFarmaco();

	ok = readField(farmaco->nome, SEPARATOR, file) == SEPARATOR;
	ok = ok && fscanf(file, "%c;", &originale) == 1;
	if (ok)
	{
		switch (originale)
		{
		case 'O':
			farmaco->originale = true;
			break;
		case 'G':
			farmaco->originale = false;
			break;
		default:
			ok = false;
			break;
		}
	}
	ok = ok && readField(farmaco->principioAttivo.nome, SEPARATOR, file) == SEPARATOR;
	ok = ok && fscanf(file, "%d;", &farmaco->principioAttivo.dosaggio);
	ok = ok && readField(farmaco->produttore, SEPARATOR, file) == SEPARATOR;
	ok = ok && fscanf(file, "%d\n", &farmaco->giacenza);

	if (ok)
	{
		return farmaco;
	}
	else
	{
		destroyFarmaco(farmaco);
		return NULL;
	}
}

void writeFarmacoToTxt(FILE *file, Farmaco *farmaco)
{
	fprintf(file, "%s;%c;%s;%d;%s;%d\n", farmaco->nome,
		farmaco->originale ? 'O' : 'G',
		farmaco->principioAttivo.nome,
		farmaco->principioAttivo.dosaggio,
		farmaco->produttore,
		farmaco->giacenza);
}

void printFarmaco(Farmaco *farmaco)
{
	writeFarmacoToTxt(stdout, farmaco);
}

int compareFarmaco(Farmaco *f1, Farmaco *f2)
{
	int comparation = comparePrincipioAttivo(f1->principioAttivo, f2->principioAttivo, false);
	if (comparation != 0)
	{
		return comparation;
	}
	if (f1->originale ^ f2->originale)
	{
		if (f1->originale)
			return -1;
		else
			return 1;
	}
	return strcmp(f1->nome, f2->nome);
}

void strtoupper(char str[])
{
	int i;
	for(i = 0; str[i] != '\0'; i++)
	{
		str[i] = toupper(str[i]);
	}
}

Boolean farmacoNomeStartsWith(Farmaco *f1, NomeFarmaco nome)
{
	unsigned int charCount = strlen(nome);
	if (strlen(f1->nome) >= charCount)
	{
		NomeFarmaco upperF1, upperNome;

		strcpy(upperNome, nome);
		strtoupper(upperNome);

		strcpy(upperF1, f1->nome);
		strtoupper(upperF1);
		return strncmp(upperF1, upperNome, charCount) == 0;
	}
	return false;
}

int comparePrincipioAttivo(PrincipioAttivo p1, PrincipioAttivo p2, Boolean ignoreDosaggio)
{
	int paComparation = strcmp(p1.nome, p2.nome);
	if (paComparation != 0 || ignoreDosaggio)
	{
		return paComparation;
	}
	else
	{
		return p1.dosaggio - p2.dosaggio;
	}
}