#include <stdlib.h>
#include "farmacoList.h"

FarmacoListItem* createFarmacoListItem(Farmaco *f)
{
	FarmacoListItem *item = (FarmacoListItem*)malloc(sizeof(FarmacoListItem));
	item->farmaco = f;
	item->next = NULL;
	return item;
}

void destroyFarmacoListItem(FarmacoListItem *item, Boolean destroyPayload)
{
	if (destroyPayload && item->farmaco != NULL)
	{
		free(item->farmaco);
	}
	free(item);
}

FarmacoList addToFarmacoList(FarmacoList list, Farmaco *f)
{
	FarmacoListItem *fli;
	FarmacoList prev = NULL, current = list;
	fli = createFarmacoListItem(f);	
	while (current != NULL && compareFarmaco(current->farmaco, f) < 0)
	{
		prev = current;
		current = current->next;
	}
	fli->next = current;
	if (prev != NULL)
	{
		prev->next = fli;
		return list;
	}
	else
	{
		return fli;
	}
}

void destroyFarmacoList(FarmacoList list, Boolean destroyPayload)
{
	if (list != NULL)
	{
		if (list->next != NULL)
		{
			destroyFarmacoList(list->next, destroyPayload);
		}
		destroyFarmacoListItem(list, destroyPayload);
	}
}

FarmacoList findByPartialNome(FarmacoList list, NomeFarmaco nome)
{
	FarmacoList outlist = NULL;
	while (list != NULL)
	{
		if (farmacoNomeStartsWith(list->farmaco, nome))
		{
			outlist = addToFarmacoList(outlist, list->farmaco);
		}
		list = list->next;
	}
	return outlist;
}

FarmacoList findByPrincipioAttivo(FarmacoList list, PrincipioAttivo pAttivo, Boolean ignoreDosaggio)
{
	FarmacoList outlist = NULL;
	while (list != NULL)
	{
		if (comparePrincipioAttivo(list->farmaco->principioAttivo, pAttivo, ignoreDosaggio) == 0)
		{
			outlist = addToFarmacoList(outlist, list->farmaco);
		}
		list = list->next;
	}
	return outlist;
}

FarmacoList readFarmacoListFromTxt(FILE *f)
{
	FarmacoList list = NULL;
	Farmaco *farmaco;
	while ((farmaco = readFarmacoFromTxt(f)) != NULL)
	{
		list = addToFarmacoList(list, farmaco);
	}
	return list;
}

void writeFarmacoListToTxt(FILE *f, FarmacoList list)
{
	while (list != NULL)
	{
		writeFarmacoToTxt(f, list->farmaco);
		list = list->next;
	}
}

void printFarmacoList(FarmacoList list)
{
	while (list != NULL)
	{
		printFarmaco(list->farmaco);
		list = list->next;
	}
}